<?php

namespace Drp\SimpleCms\Contracts;

/**
 * Interface Publishable
 * @package Drp\SimpleCms
 */
interface Publishable
{
    const STATUS_DRAFT = 'Draft';

    const STATUS_REVIEW = 'In review';

    const STATUS_PUBLISHED = 'Published';

    const STATUS_UNPUBLISHED = 'Un-published';

    const STATUSES = [
        self::STATUS_DRAFT,
        self::STATUS_REVIEW,
        self::STATUS_PUBLISHED,
        self::STATUS_UNPUBLISHED,
    ];

    /**
     * Generate a unique Id for the content.
     *
     * @return integer
     */
    public function generateContentId();

    /**
     * Return if the current content has a status of published.
     *
     * @return boolean
     */
    public function isPublished();

    /**
     * Return if the current content has a status of un-published.
     *
     * @return boolean
     */
    public function isUnPublished();

    /**
     * Return if the current content is a draft copy.
     *
     * @return boolean
     */
    public function isDraft();

    /**
     * Return if the current content is marked as in review.
     *
     * @return boolean
     */
    public function isInReview();

    /**
     * Determine if viewing draft content.
     *
     * @return boolean
     */
    public function isDraftOrInReview();

    /**
     * Publish the content if it is a draft.
     * Un-publish the content if it is published (set to Draft)
     *
     * @return void
     */
    public function updatePublishStatus();

    /**
     * Update the content to be in-review if it's a draft.
     * Update to draft it is already in-review
     *
     * @return void
     */
    public function updateReviewStatus();

    /**
     * Return if the current content has been saved to the DB so can be published.
     *
     * @return boolean
     */
    public function canBePublished();

    /**
     * Return if the current content is can be reviewed.
     * Only draft and in review status can be reviewed.
     *
     * @return boolean
     */
    public function canBeReviewed();

    /**
     * Return if the content can be reset. This means it
     * is currently draft and has a published
     * version to fall back to
     *
     * @return boolean
     */
    public function canBeReset();

    /**
     * Return if published version of the current content exists.
     *
     * @return boolean
     */
    public function hasPublishedVersion();

    /**
     * Return if a draft|awaiting version of the current content exists.
     *
     * @return boolean
     */
    public function hasDraftOrReviewVersion();

    /**
     * Return if a unpublished version of the current content exists.
     *
     * @return boolean
     */
    public function hasUnPublishedVersion();


    /**
     * Remove the draft version of the content so there is only the published version left.
     *
     * @return mixed
     * @throws \Drp\SimpleCms\Exceptions\CouldNotResetDraft If a draft version or the draft version is the only version.
     */
    public function resetDraft();

    /**
     * Get all versions of the Content.
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function allVersionsOfContent();

    /**
     * Return the latest version of the content using the given Id.
     * If their is not a draft version the published version will be returned.
     * If there is then that draft will be returned.
     *
     * @param integer $contentId
     * @return Content
     */
    public static function getLatestVersionById($contentId);

    /**
     * Return the published version of the current content.
     *
     * @throws \Illuminate\Database\Eloquent\ModelNotFoundException Thrown if no published version exists.
     * @return static
     */
    public function getPublishedVersion();

    /**
     * Return the draft version of the current content.
     *
     * @throws \Illuminate\Database\Eloquent\ModelNotFoundException Thrown if no draft version exists.
     * @return static
     */
    public function getUnpublishedVersion();

    /**
     * Return the draft|awaiting version of the current content.
     *
     * @throws \Illuminate\Database\Eloquent\ModelNotFoundException Thrown if no draft version exists.
     * @return static
     */
    public function getDraftOrReviewVersion();

    /**
     * Return a closure that is called before a study is saved.
     *
     * @return \Closure|null
     */
    public function beforeSave();

    /**
     * Return a closure that is called after a study is saved.
     *
     * @return \Closure|null
     */
    public function afterSave();

    /**
     * Return all the content. Only include the
     * Published content or content that only have a draft
     * i.e ones that have never been published.
     *
     * @param array $with Optional relationships to eager load
     * @return mixed
     */
    public static function getAllVersionsPrioritisedByStatus(array $with = []);

    /**
     * Check if the content has changed from the DB version when the attributes are applied
     *
     * @param mixed $attributes
     * @return mixed
     */
    public function isContentDirty($attributes);
}
