<?php

namespace Drp\SimpleCms\Test\Unit;

use Drp\SimpleCms\ContentManager;
use Drp\SimpleCms\Contracts\Publishable;
use Drp\SimpleCms\Exceptions\NoUpdatesMade;
use Drp\SimpleCms\Test\Support\Fakes\TestContent;
use Drp\SimpleCms\Test\TestCase;

class ContentManagerTest extends TestCase
{

    /**
     * @test
     */
    public function it_will_create_a_new_content()
    {
        $manager = $this->app->make(ContentManager::class);
        $manager->createContent('test-content', ['title' => 'Content']);
        $this->seeInDatabase('test_contents', [
            'title' => 'Content',
            'status' => Publishable::STATUS_DRAFT,
            'content_id' => '1',
        ]);
    }

    /**
     * @test
     */
    public function it_will_update_a_draft_copy_if_the_content_has_not_yet_been_published()
    {
        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->make(['title' => 'Title']);

        $manager = $this->app->make(ContentManager::class);
        $manager->updateContent($content, ['title' => 'Updated content']);

        $this->assertCount(1, TestContent::withoutGlobalScopes()->get());

        $this->dontSeeInDatabase('test_contents', [
            'id' => $content->id,
            'title' => 'Title',
            'status' => Publishable::STATUS_DRAFT
        ]);

        $this->seeInDatabase('test_contents', [
            'id' => $content->id,
            'title' => 'Updated content',
            'status' => Publishable::STATUS_DRAFT
        ]);
    }

    /**
     * @test
     */
    public function it_will_create_a_draft_copy_if_the_content_exists()
    {
        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create(['title' => 'Title']);

        $manager = $this->app->make(ContentManager::class);
        $draft = $manager->updateContent($content, ['title' => 'Updated content']);

        $this->assertCount(2, TestContent::get());
        $this->seeInDatabase('test_contents', ['content_id' => $content->content_id, 'status' => Publishable::STATUS_PUBLISHED]);
        $this->seeInDatabase('test_contents', ['content_id' => $content->content_id, 'title' => 'Updated content', 'status' => Publishable::STATUS_DRAFT]);
    }

    /**
     * @test
     */
    public function it_will_create_a_draft_copy_if_the_content_exists_but_is_unpublished()
    {
        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_UNPUBLISHED)
            ->create(['title' => 'Title']);

        $manager = $this->app->make(ContentManager::class);
        $draft = $manager->updateContent($content, ['title' => 'Updated content']);

        $this->assertCount(2, TestContent::get());
        $this->seeInDatabase('test_contents', ['id' => $content->id, 'status' => Publishable::STATUS_UNPUBLISHED]);
        $this->seeInDatabase('test_contents', ['id' => $draft->id, 'title' => 'Updated content', 'status' => Publishable::STATUS_DRAFT]);
    }

    /**
     * @test
     */
    public function it_will_update_a_draft_copy_if_the_content_is_in_draft_mode()
    {
        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create(['title' => 'Title']);

        $manager = $this->app->make(ContentManager::class);
        $draft = $manager->updateContent($content, ['title' => 'Updated content']);

        $manager->updateContent($content, ['title' => 'Second updated content']);

        $this->assertCount(2, TestContent::get());
        $this->seeInDatabase('test_contents', ['id' => $content->id, 'status' => Publishable::STATUS_PUBLISHED]);
        $this->seeInDatabase('test_contents', ['id' => $draft->id, 'title' => 'Second updated content', 'status' => Publishable::STATUS_DRAFT]);
    }

    /**
     * @test
     */
    public function it_will_only_update_its_own_draft_copy()
    {
        $firstContent = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create(['title' => 'Title']);

        $firstContentDraft = factory(TestContent::class)->states(Publishable::STATUS_DRAFT)->create([
            'content_id' => $firstContent->content_id,
        ]);

        $secondContent = factory(TestContent::class)->states(Publishable::STATUS_PUBLISHED)->create([

            'title' => 'My title'
        ]);

        $manager = $this->app->make(ContentManager::class);
        $manager->updateContent($secondContent, ['title' => 'Second updated content']);

        $this->seeInDatabase('test_contents', ['id' => $firstContent->id, 'title' => $firstContent->title]);
        $this->seeInDatabase('test_contents', ['id' => $firstContentDraft->id, 'title' => $firstContentDraft->title]);
        $this->seeInDatabase('test_contents', ['id' => $secondContent->id, 'title' => 'My title']);
        $this->seeInDatabase('test_contents', ['content_id' => $secondContent->content_id, 'title' => 'Second updated content']);
    }

    /**
     * @test
     */
    public function the_before_save_callback_is_called()
    {
        $content = new TestContent(['title' => 'Content']);
        $timesCallbackRan = 0;

        $content->beforeCallback = function () use (&$timesCallbackRan) {
            $timesCallbackRan++;
        };

        $manager = $this->app->make(ContentManager::class);
        $manager->updateContent($content, ['title' => 'Updated content']);
        $this->assertEquals(1, $timesCallbackRan);
    }

    /**
     * @test
     */
    public function the_after_save_callback_is_called()
    {
        $content = new TestContent(['title' => 'Content']);
        $timesCallbackRan = 0;

        $content->afterCallback = function () use (&$timesCallbackRan) {
            $timesCallbackRan++;
        };

        $manager = $this->app->make(ContentManager::class);
        $manager->updateContent($content, ['title' => 'Updated content']);
        $this->assertEquals(1, $timesCallbackRan);
    }

    /**
     * @test
     */
    public function it_does_not_a_create_unique_slug_if_updating_same_content()
    {
        $this->markTestSkipped();

        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create(['title' => 'Title']);

        $draftContent = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create([
                'content_id' => $content->content_id,
                'title' => 'Title',
            ]);

        $this->assertEquals($content->slug, $draftContent->slug);
    }

    /**
     * @test
     */
    public function it_has_the_same_slug_if_creating_a_new_published_content_with_same_title_as_a_existing_published_content()
    {
        $this->markTestSkipped();

        $contentA = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create([
                'title' => 'Steve Newbury',
            ]);

        $contentB = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create([
                'title' => 'Steve Newbury',
            ]);

        $this->assertEquals('steve-newbury', $contentA->slug);
        $this->assertEquals('steve-newbury', $contentB->slug);
    }

    /**
     * @test
     */
    public function it_has_the_same_slug_if_creating_a_new_draft_content_with_same_title_as_a_existing_draft_content()
    {
        $this->markTestSkipped();

        $draftContent = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create([
                'title' => 'Luke Renfrew',
            ]);

        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create([
                'title' => 'Luke Renfrew',
            ]);

        $this->assertEquals('luke-renfrew', $draftContent->slug);
        $this->assertEquals('luke-renfrew', $content->slug);
    }

    /**
     * @test
     */
    public function it_does_no_create_a_unique_slug_if_when_creating_for_same_content()
    {
        $this->markTestSkipped();

        $contentA = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create([
                'title' => 'Luke Renfrew',
            ]);

        $contentB = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create([
                'content_id' => $contentA->content_id,
                'title' => 'Luke Renfrew',
            ]);

        $this->assertEquals('luke-renfrew', $contentA->slug);
        $this->assertEquals('luke-renfrew', $contentB->slug);
    }


    /**
     * @test
     */
    public function an_exception_is_thrown_if_no_updated_where_made_to_the_content()
    {
        try {
            $content = factory(TestContent::class)->create();
            $manager = $this->app->make(ContentManager::class);
            $manager->updateContent($content, []);
        } catch (NoUpdatesMade $exception) {
            $this->assertCount(1, TestContent::all());
            return;
        }

        $this->fail('A draft was created event when no updated where made');
    }
}
